// Rotary encoder on SPI bus for 9Xtreme

unsigned char EncoderData[2] ;
unsigned char LastEncoderPosition ;

uint16_t MillisPrecount ;
uint16_t lastTimerValue ;
uint32_t TotalMicros ;
uint32_t TotalMillis ;
uint8_t Correction ;

#define SCK_PIN   13  // D13 = pin19 = PortB.5
#define MISO_PIN  12  // D12 = pin18 = PortB.4
#define MOSI_PIN  11  // D11 = pin17 = PortB.3
#define SS_PIN    10  // D10 = pin16 = PortB.2

void SlaveInit(void)
{
	// Set MISO output, all others input
	pinMode(SCK_PIN, INPUT);
	pinMode(MOSI_PIN, INPUT);
	pinMode(MISO_PIN, OUTPUT);  // (only if bidirectional mode needed)
	pinMode(SS_PIN, INPUT);

 /*  Setup SPI control register SPCR
 SPIE - Enables the SPI interrupt when 1
 SPE - Enables the SPI when 1
 DORD - Sends data least Significant Bit First when 1, most Significant Bit first when 0
 MSTR - Sets the Arduino in master mode when 1, slave mode when 0
 CPOL - Sets the data clock to be idle when high if set to 1, idle when low if set to 0
 CPHA - Samples data on the trailing edge of the data clock when 1, leading edge when 0
 SPR1 and SPR0 - Sets the SPI speed, 00 is fastest (4MHz) 11 is slowest (250KHz)   */
 
 // enable SPI subsystem and set correct SPI mode
	SPCR = (1<<SPIE)|(1<<SPE)|(0<<DORD)|(0<<MSTR)|(1<<CPOL)|(1<<CPHA)|(0<<SPR1)|(1<<SPR0);
}

void init()
{
	// Timer1
  TCCR1A = 0x00 ;    //Init.
  TCCR1B = 0xC1 ;    // I/p noise cancel, rising edge, Clock/1
}

void setup()
{
// For 16MHz, uncomment next 3 lines
//	cli() ;
//	CLKPR = 0x80 ;
//	CLKPR = 1 ;
	sei() ;
  pinMode( 2, INPUT_PULLUP ) ;	// PD2
  pinMode( 3, INPUT_PULLUP ) ;	// PD3
  pinMode( 4, INPUT_PULLUP ) ;	// PD4
	LastEncoderPosition = (PIND >> 2) & 3 ;
	SlaveInit() ;
}

void loop()
{
  unsigned char pins ;
  unsigned char position ;

  pins = PIND ;
	pins >>= 2 ;
	EncoderData[1] = ( pins & 4 ) ? 0 : 1 ;

	pins &= 3 ;
	if ( pins != LastEncoderPosition )
	{
		unsigned char x ;
		x = pins & 2 ;
		x >>= 1 ;
		x ^= LastEncoderPosition & 1 ;
		position = EncoderData[0] ;
		if ( x )
		{
			position -= 1 ;
		}
		else
		{
			position += 1 ;
		}
		EncoderData[0] = position ;

		LastEncoderPosition = pins ;
	}
  delayMicroseconds(50) ;
}


ISR(SPI_STC_vect)
{
	uint8_t x ;
	uint16_t y ;
	x = SPDR ;
	if ( x == 0x5A )
	{
		SPDR = 0xAA | ( EncoderData[1] & 1 ) ;
		y = 0 ;
		while(!(SPSR & (1<<SPIF)))
		{
			if ( ++y > 500 )
			{
				return ;
			}
		}
		SPDR = EncoderData[0] ;
		y = 0 ;
		while(!(SPSR & (1<<SPIF)))
		{
			if ( ++y > 500 )
			{
				return ;
			}
		}
		x = SPDR ;
		SPDR = EncoderData[0] ;
	}
}

int main()	// run over and over again
{
  init() ;
  setup() ;
	for(;;)
	{
		loop() ;
	}
}

uint32_t micros()
{
	uint16_t elapsed ;
	uint8_t millisToAdd ;
	uint8_t oldSREG = SREG ;
	cli() ;
	uint16_t time = TCNT1 ;	// Read timer 1
	SREG = oldSREG ;

	elapsed = time - lastTimerValue ;
	elapsed += Correction ;
	
// For 8MHz
	Correction = elapsed & 0x07 ;
	elapsed >>= 3 ;
	 
	uint32_t ltime = TotalMicros ;
	ltime += elapsed ;
	cli() ;
	TotalMicros = ltime ;	// Done this way for RPM to work correctly
	lastTimerValue = time ;
	SREG = oldSREG ;	// Still valid from above
	
	elapsed += MillisPrecount;
	millisToAdd = 0 ;
	if ( elapsed  > 3999 )
	{
		millisToAdd = 4 ;
		elapsed -= 4000 ;
	}
	else if ( elapsed  > 2999 )
	{
		millisToAdd = 3 ;		
		elapsed -= 3000 ;
	}
	else if ( elapsed  > 1999 )
	{
		millisToAdd = 2 ;
		elapsed -= 2000 ;
	}
	else if ( elapsed  > 999 )
	{
		millisToAdd = 1 ;
		elapsed -= 1000 ;
	}
	TotalMillis += millisToAdd ;
	MillisPrecount = elapsed ;
	return TotalMicros ;
}

uint32_t millis()
{
	micros() ;
	return TotalMillis ;
}

void delay(unsigned long ms)
{
	uint16_t start = (uint16_t)micros();
	uint16_t lms = ms ;

	while (lms > 0) {
		if (((uint16_t)micros() - start) >= 1000) {
			lms--;
			start += 1000;
		}
	}
}
  
void delayMicroseconds(uint16_t x)
{
	uint16_t start = (uint16_t)micros();
	while (((uint16_t)micros() - start) < x )
	{
	}	
}

